#!/usr/bin/perl -w

#
# $Id: check_nt_perfmon_counter.pl / Version 1.01 / 2012-11-19 14:36 / projects@thesysadmin.net
#
# Copyright (C) 2010 Martin Mueller,
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# Report bugs to:  projects@thesysadmin.net
#
#
# CHANGELOG:
#
# 1.00 - 08.09.2010 - First public version
# 1.01 - 19.11.2012 - Small Bugfix (now it works also, if no password is set for check_nt)
#					  (Thanks to Stefan)
#

$ENV{'PATH'}='';
$ENV{'BASH_ENV'}='';
$ENV{'ENV'}='';

use POSIX;
use warnings;
use strict;


# Find the Nagios::Plugin lib. Change this path if necessary
use FindBin;
use lib "$FindBin::Bin/../perl/lib";
use Nagios::Plugin qw(%ERRORS);

use Getopt::Long qw(:config no_ignore_case bundling);

# define Constants

use vars qw($PROGRAMNAME $SHORTNAME $AUTHOR $COPYR $VERSION);
$PROGRAMNAME = "$FindBin::Script";
$SHORTNAME = "CheckNT PerfmonCounter";
$AUTHOR = "Martin Mueller";
$COPYR = "Copyright (C) 2010";
$VERSION = "Version: 1.01";

sub usage();
sub help();

# define Commandline-Options

my $host = undef;
my $port = 1248;
my $binarypath = undef;
my $temppath = "/tmp/";
my $secret = "";
my $type = undef;
my $warning = undef;
my $warning_up = undef;
my $critical = undef;
my $critical_up = undef;
my $timeout = 15;
my $help = undef;
my $printversion = undef;


my $np = Nagios::Plugin->new(
                shortname => "$SHORTNAME",
);

# get Options

GetOptions(
   "H|host=s"           => \$host,
   "p|port=i"   		=> \$port,
   "s|secret=s"			=> \$secret,
   "B|binarypath=s"		=> \$binarypath,
   "P|temppath=s"		=> \$temppath,
   "T|type=s"			=> \$type,
   "w|warning=i"        => \$warning,
   "W|warning-up=i"   	=> \$warning_up,
   "c|critical=i"       => \$critical,
   "C|critical-up=i"	=> \$critical_up,
   "t|timeout=i"        => \$timeout,
   "h|help"             => \$help,
   "V|version"          => \$printversion,
   );

if ($help) {
	help();
	}

if ($printversion) {
	printf "\n";
	printf "$PROGRAMNAME - $VERSION\n\n";
	printf "$COPYR $AUTHOR\n";
	printf "This programm comes with ABSOLUTELY NO WARRANTY\n";
	printf "This programm is licensed under the terms of the GNU General Public License";
	printf "\n\n";
	exit($ERRORS{'UNKNOWN'});
	}

if (!defined $host) {
	printf "\nMissing argument [host]. Please specify a hostname or ip address\n";
	usage();
	}

if (!defined $type) {
        printf "\nMissing argument [type]. Please specify which counter you want to check\n";
        usage();
        }

if (!defined $binarypath) {
	printf "\nMissing argument [path]. Please specify path to you 'check_nt' binary\n";
	usage();
	}

# Just in case of problems, let's not hang Nagios
$SIG{'ALRM'} = sub {
	$np->nagios_die("No response from $host (alarm)");
	};

alarm($timeout);

# -----------------------------------------------------
# Define Variables
# -----------------------------------------------------

my $check_binary = "check_nt";
(my $trash, my $name1, my $name2) = split (/\\/, $type);
my $temp_file = $host."_".$name1."_".$name2;

my $current_counter = undef;
my $old_counter = undef;
my $result = 0;
my $errorcode = undef;

# ------------------------------------------------------
# Start here with Main Program
# ------------------------------------------------------

$current_counter = &get_perfmondata();

if ($current_counter=~/\D/) {
        $np->nagios_exit('UNKNOWN',"Check given Parameter 'Type'. Output isn't numeric");
        }

#Check if Temp-File exists. Otherwise create a new one and exit

if (-e $temppath.$temp_file) {
	$old_counter = &get_old_counter();
	}
else {
	&write_tempfile();
	$np->nagios_exit('UNKNOWN',"Temp-File with old Value does not exist, creating new one...");
	}

if ($current_counter < $old_counter) {
	&write_tempfile();
        $np->add_message('UNKNOWN',"Temp-File has old Value > current Value, has the Server restarted? updating file...");
	}
elsif ($current_counter > $old_counter) {
	$result = $current_counter-$old_counter;
	&write_tempfile();
	if (defined ($critical) && defined ($critical_up)) {
		if ($critical>=$result || $critical_up<=$result) {
			$errorcode = "CRITICAL";
			}
		elsif ($warning>=$result || $warning_up<=$result) {
			$errorcode = "WARNING";
			}
		else {
			$errorcode = "OK";
			}
		}	
	elsif (defined ($critical)) {
		if ($critical>=$result) {
			$errorcode = "CRITICAL";
			}
		elsif ($warning>=$result) {
        	        $errorcode = "WARNING";
	                }
		else {
			$errorcode = "OK";
			}
		}
	elsif (defined ($critical_up)) {
		if ($critical_up<=$result) {
			$errorcode = "CRITICAL";
			}
		elsif ($warning_up<=$result) {
			$errorcode = "CRITICAL";
			}
		else {
			$errorcode = "OK";
			}
		}
	else {
		$errorcode = "OK";
		}
	$np->add_message($errorcode,$name1."_".$name2." = ".$result);
	}
elsif ($current_counter == $old_counter) {
	$np->add_message('CRITICAL',"Temp-File has old Value == current Value, is everything OK?");
	}
else {
	# something went wrong
	printf "should not see me...\n";
	$np->add_message('UNKNOWN',"should not see me");
	}

$np->add_perfdata(
	label => $name1."_".$name2,
	value => $result,
	);


my ($code, $message) = $np->check_messages(join => "<BR>",join_all => "<BR>");
$np->nagios_exit($code,$message);

# ------------------------------------------------------
# End Main Program
# ------------------------------------------------------

sub exit_plugin() {
	my ($code, $message) = $np->check_messages(join => "<BR>",join_all => "<BR>");
	$np->nagios_exit($code,$message);
	}

sub get_perfmondata() {
	my $check_value;
	if ($secret) {
		$check_value = `$binarypath$check_binary -p $port -s $secret -H $host -v COUNTER -l '$type'`;
		}
	else {
		$check_value = `$binarypath$check_binary -p $port -H $host -v COUNTER -l '$type'`;
	}
	chomp($check_value);
	$np->nagios_die("Unable to execute check_nt. Please check given parameters (ie port, secret, etc)") if ($check_value=~/^[a-z,A-Z]/);
	alarm(0);
	return $check_value;
	}

sub get_old_counter() {
	my $tempfile_value=undef;
	my $line=undef;
	open(TEMPFILE, "$temppath$temp_file");
	my @old_value=<TEMPFILE>;
	close (TEMPFILE);
	foreach $line (@old_value) {
		$tempfile_value = $line;
		chomp($tempfile_value);
		}
	alarm(0);
	return $tempfile_value;
	}

sub write_tempfile() {
	open(TEMPFILE, ">$temppath$temp_file");
        print TEMPFILE "$current_counter";
        close (TEMPFILE);
	}

sub usage () {
	printf "\n";
	printf "\n$PROGRAMNAME $VERSION\n";
	printf "$COPYR $AUTHOR\n";
	printf "This programm comes with ABSOLUTELY NO WARRANTY\n";
	printf "This programm is licensed under the terms of the GNU General Public License\n\n";
	printf "$PROGRAMNAME -H <hostname> [-p <port>] [-s <secret>] -P <path to your check_nt binary> -T <type>\n\n";
	printf "For detailed help use: $PROGRAMNAME --help\n\n";
	exit($ERRORS{'UNKNOWN'});
	}

sub help () {
	printf "\n\n$PROGRAMNAME $VERSION - plugin for Nagios \n";
	printf "this plugin uses check_nt to query the specified counter and calculates the difference between the last check and the current one\n";
	printf "To use this Plugin you need check_nt installed on your Nagios-Host and NSClient, NC-Net, etc on your Windows-Machine\n";
	printf "You can use parameters [-w] [-c] with [-C] [-W] together to define a range\n";
	printf "\nUsage:\n";
	printf "   -H (--hostname)     Hostname to query - (required)\n";
	printf "   -p (--port)	       Port (default=1248)\n";
	printf "   -B (--binarypath)   Path to your check_nt binary - (required) (ex: /usr/local/nagios/libexec/)\n";
	printf "   -P (--temppath)     Path to your temp directory (default=/tmp/)\n";
	printf "   -s (--secret)       Specify a secret if defined to connect to NSClient, NC-Net\n";
	printf "   -T (--type)         Specify a check_nt compatible expression which counter you want to check - (required)\n";
	printf "                         Examples:  (important. You have to define 2 Backslashes (one will be killed by the script ;-) )\n";
	printf "                           [-T \"\\\\MSExchangeTransport SmtpSend(organisationsinterner smtp-sendeconnector)\\\\Messages Sent Total\"\n";
	printf "   -w (--warning)      Warning threshold (checkresult < warning) - (requires parameter [-c])\n";
	printf "   -W (--warning-up)   Warning threshold (checkresult > warning) - (requires paramter [-C]\n";
	printf "   -c (--critical)     Critical threshold (checkresult < critical)\n";
	printf "   -C (--critical-up)  Critical threshold (checkresult > critical)\n";
	printf "   -t (--timeout)      Seconds before the plugin times out (default=15)\n";
	printf "   -V (--version)      Plugin version\n";
	printf "   -h (--help)         Usage help \n\n";
	exit($ERRORS{'OK'});
}

